#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <server.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/stat.h>

#include <errExit.h>
#include <signal.h>
#include <custom_msgq.h>
#include <custom_sem.h>
#include <custom_shm.h>
#include <custom_sig.h>
#include <forza4.h>

input_server_t _INPUT_S;

input_server_t checkServerInput(int argc, char *argv[]);

void setupServer(int argc, char *argv[]){
    // CHECK_INPUT
    _INPUT_S = checkServerInput(argc, argv);

    _ROWS = _INPUT_S.rows;
    _COLLUMS = _INPUT_S.collums;

    // MSGQ
    printf(PSERVER "Setting up msgq\n");
    setupServerMsgq();
    
    // SIGNAL
    printf(PSERVER "Setting up signals\n");
    setupServerSignalHandler();

    // SEM
    printf(PSERVER "Setting up sem\n");
    setupServerSem();

    // SHM
    printf(PSERVER "Setting up shm\n");
    setupServerShm();
}


input_server_t checkServerInput(int argc, char *argv[]){
    if (argc < 5) {
        printf("\033[92m<Help>\033[39m ./F4Server ROW COL PLAYER1 PLAYER2\n"
               "\t- ROW \t\tnumber of rows\n"
               "\t- COL \t\tnumber of rows\n"
               "\t- PLAYER1 \tplayer one token\n"
               "\t- PLAYER2 \tplayer two token\n"
               );
        exit(EXIT_SUCCESS);
    } else if (argc > 5) {
        errExitMsg("To many arguments");
    }

    input_server_t input = {
        .collums = atoi(argv[1]),
        .rows = atoi(argv[2]),
        .player1Token = argv[3][0],
        .player2Token = argv[4][0]
    };

    if (input.collums<5 || input.rows<5) {
        errExitMsg("To few rows or collums");
    }

    if (!isprint(input.player1Token) || !isprint(input.player2Token)) {
        errExitMsg("Invalid player token");
    }

    return input;
}